module Clarc
  class FormBase
    include Virtus.model

    attribute :errors, nil, default: []
    attribute :params

    def initialize params
      @params = params
      build
    end

    def validators
      {}
    end

    def valid?
      @errors = {}
      valid = true

      validators.each do |attr, klass|
        value = send(attr)
        if value.is_a?(Array)
          value.each_with_index do |value, i|
            validator = klass.new(value)
            valid &= validator.valid?

            @errors[attr] ||= {}
            @errors[attr][i] = validator.errors
          end
        else
          validator = klass.new(value)
          valid &= validator.valid?
          @errors[attr] = validator.errors
        end
      end

      valid
    end

    def to_hash
      result = { errors: @errors }
      validators.each{ |attr, _| result[attr] = send(attr) }
      result
    end

    def persist!
      validators.each do |attr, _|
        model_name = attr.to_s.camelize
        repository = "#{model_name}Repository".constantize
        repository.save(send(attr))
      end
    end
  end
end
